clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Read and process friction data new friction tester to show measured
% normal forces during measurement

% Version:
%   v1: 23-3-2023

% Matlab version: R2022b 


%% initialisation

serie = 104; % measurement series
meas = 12;

figure(1), clf(1)
figure(2), clf(2)

%% load data

% find file and get data
cd ../FT_C-LMPAEK_15kPa_365degC/
file_find = dir(['*',num2str(serie),'-',num2str(meas),'.txt']);
file_name = file_find.name; 
data = readmatrix(file_name);
fprintf(['File: ', file_find.name,'\n'])
cd ../Scripts

%% interpret data

t = data(:,1) - data(1,1); % time [s]
d = data(:,2) - data(1,2); % displacement [mm]
F_pull = data(:,3); % pull force [N]
F_M = data(:,4); % normal force middle [N]
F_L = data(:,5); % normal force left [N]
F_R = data(:,6); % normal force right [N]

%% compute shear stress

A = 100*50; % contact area, mm^2      
tau = F_pull/(2*A)*1e3; % shear stress; 2 slip interfaces, from MPa to kPa 

%% determine start and end points

% velocity from file name
name_u = find(file_find.name=='u');
name_u_end = name_u + 1;
while file_find.name(name_u_end) ~= '_'
    name_u_end = name_u_end + 1;
end
vel = str2double(file_find.name(name_u+1:name_u_end-1)); % velocity from file name

% determine window for smoothening based on velocity from file name
if vel <= 1
    window = 525;
elseif vel <= 3
    window = 300;
elseif vel <= 5
    window = 150;
elseif vel <= 11
    window = 50;
elseif vel <= 15
    window = 45;
elseif vel <= 30
    window = 40;
elseif vel <= 50
    window = 25;
else
    window = 15;
end

% smoothening data
d_filt = smoothdata(d,'gaussian',window); % filter data met moving median filter

v_vec = [0]; % velocity vector
v_vec_1 = [0]; % velocity vector 
a_vec = [0]; % acceleration vector
for i = 2:length(t)-1
    dt_L = t(i) - t(i-1);
    dt_R = t(i+1) - t(i);
    v = (d_filt(i+1)-d_filt(i-1))/(dt_R+dt_L);
    a = (d_filt(i+1)-2*d_filt(i)+d_filt(i-1))/(0.5*(dt_R^2+dt_L^2)); % actually: error due to f' times difference in time step
    v_vec(i) = v;
    a_vec(i) = a;

    % first order derivative
    v_vec_1(i) = (d_filt(i+1)-d_filt(i))/(t(i+1)-t(i));
end
v_vec(length(t)) = v;
v_vec_1(length(t)) = v_vec_1(length(t)-1);
v_vec_grad = gradient(d_filt(:)) ./ gradient(t(:));
a_vec(length(t)) = a;

% smooth data at very low V
if vel == 1
    a_vec = smoothdata(a_vec,'gaussian',50);
elseif vel >= 2 && vel < 5
    a_vec = smoothdata(a_vec,'gaussian',10);
elseif vel == 5
    a_vec = smoothdata(a_vec,'gaussian',5);
end

% find global start and end points
i_glob = 1; % global end range of start point
while d(i_glob) < 1 % deformation below 1 mm
    i_glob = i_glob + 1;
end

i_glob_9 = i_glob; % global end range of start point
while d(i_glob_9) < 9 % deformation around 9 mm
    i_glob_9 = i_glob_9 + 1;
end

i_glob_start = i_glob; % find around i_glob also global start range for start point
while t(i_glob_start) > t(i_glob)-(2/vel*60) && i_glob_start > 2 % move 2 mm in time earlier based on velocity in experiment
    i_glob_start = i_glob_start - 1;
end
        
i_glob_end = i_glob;
while t(i_glob_end) < t(i_glob) + d(end)/vel*60 && i_glob_end < length(t)-1 % find index for time after full displacement
    i_glob_end = i_glob_end + 1;
end
idx_start = find(a_vec(i_glob_start:i_glob)==max(a_vec(i_glob_start:i_glob)))+i_glob_start-1;
idx_end = find(a_vec(i_glob_9:i_glob_end)==min(a_vec(i_glob_9:i_glob_end)))+i_glob_9-1;

%% plot data

figure(1), hold on
plot(t,F_M,'-','displayname',['Middle'],'linewidth',1.0,'Color',[0.00 0.45 0.74])
plot(t,F_L,'-','displayname',['Left'],'linewidth',1.0,'Color',[0.85 0.33 0.1])
plot(t,F_R,'-','displayname',['Right'],'linewidth',1.0,'Color',[0.93 0.69 0.13])
plot([t(idx_start) t(idx_start)],[100 100],'--k','color',[0.5 0.5 0.5],'markersize',12,'displayname',['Start \& end', newline ,'sliding'],'linewidth',1.0)
plt = plot([50 50 60 60 50],[54 57 57 54 54],'-k');
set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );
plt = plot([72.8 60],[43.3 57],'-k');
set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );
plt = plot([10.8 50],[43.3 57],'-k');
set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );

% schematic location compression force cells
x_shift = 5;
y_shift = 2;
plot([88 112 112 88 88]-x_shift,[28 28 20 20 28]-y_shift,'-k','linewidth',1.0)
theta = 0:pi/50:2*pi;
x_circ = 13.2 * cos(theta) + 100 - x_shift;
y_circ = 8.8 * sin(theta) + 24 - y_shift;
plot(x_circ, y_circ,':k','linewidth',0.5)
plot([86.8 - x_shift],[24] - y_shift,'k.','markersize',20,'color',[0.00 0.45 0.74])  % middle
plot([106.6 - x_shift],[31.6] - y_shift,'k.','markersize',20,'Color',[0.85 0.33 0.1])  % left
plot([106.6 - x_shift],[16.4] - y_shift,'k.','markersize',20,'Color',[0.93 0.69 0.13])  % right
text([80],[34],'Meas. locations:','Fontsize',8,'interpreter','latex')
annotation('arrow',[0.77 0.89],[0.41 0.41],'Linewidth',0.5,'HeadStyle','cback1','HeadLength',4,'HeadWidth',4)
text([112],[24],'$V$','Fontsize',8,'interpreter','latex')

%leg = legend('box','off','fontsize',8,'interpreter','latex','location','SouthEast','NumColumns',1);
%leg.ItemTokenSize = [11,18];
xlabel('Time, $t$ [s]','interpreter','latex','fontsize',8)
ylabel('Normal force, $F_{\rm{n}}$ [N]','interpreter','latex','fontsize',8)
set(gca,'fontsize',8);
set(gcf,'Renderer','painters')

% figure size 
width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
set(gcf,'units','centimeters','position',[5,5,width,0.75*width])
axis([0 120 0 60])
grid on
box on

% create a new pair of axes inside current figure
axes('position',[.2 .3 .4 .4])
hold on
box on 
grid on
indexOfInterest = (t < 60) & (t > 50); 
plot(t(indexOfInterest),F_M(indexOfInterest),'Color',[0.00 0.45 0.74]) % plot on new axes
plot(t(indexOfInterest),F_L(indexOfInterest),'Color',[0.85 0.33 0.1]) % plot on new axes
plot(t(indexOfInterest),F_R(indexOfInterest),'Color',[0.93 0.69 0.13]) % plot on new axes
plot([t(idx_start) t(idx_start)],[52 60],'-.k','color',[0.5 0.5 0.5],'markersize',12,'displayname',['Start sliding'],'linewidth',1.0)
plot([t(idx_end) t(idx_end)],[52 60],'-.k','color',[0.5 0.5 0.5],'markersize',12,'displayname',['End sliding'],'linewidth',1.0)
text(52.1,53.71,'Start','Fontsize',8,'Interpreter','Latex')
text(57.1,53.71,'End','Fontsize',8,'Interpreter','Latex')
axis([50 60 54 57])
set(gca,'fontsize',8);
set(gcf,'Renderer','painters')

% save for PDF (as EPS doesn't support transparency)
set(gcf,'PaperUnits','centimeters','PaperSize',[width 0.75*width])
print('-painters','-dpdf','Fig_normal_forces.pdf')


% Plot shear stress response
% figure(2), hold on
% plot(d-d(1),tau,'linewidth',1.0,'displayname',['-'])
% %plot(d(idx_peak),tau_peak,'rx','markersize',12)
% %plot([d(ss_idx_start) d(ss_idx_end)],[tau_mean tau_mean],'k-','linewidth',2.0)
% xlabel('Displacement, $d$ [mm]','interpreter','latex','fontsize',16)
% ylabel('Shear stress, $\tau$ [kPa]','interpreter','latex','fontsize',16)
% legend('fontsize',14,'interpreter','latex','location','NorthEast','NumColumns',2)
% set(gca,'fontsize',16)
% grid on
% box on






