clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Compute shear stress versus displacement and corresponding stress
% relaxation after cessation of the sliding movement for several different 
% rates as measured with the new friction tester (nFT) on C/PEEK

% Version:
%   v1: 7-2-2024

% Matlab version: R2023a 


%% initialisation
rel_time = 31; % time after sliding stopped, for relaxation curve
mod_coeff = 0.9; %threshold for modulus fit, end; max difference local and average modulus

% clear figures
figure(1), clf(1)
figure(2), clf(2)

% determine measurement range
serie = 103; % measurement series
range_meas = [21 22 23 7 8 9 4 5 6 1 2 3]; %[1 2 3 4 5 6 7 8 9 13 14 15 16 17 18 19 20 21 22 23]; % measurements in serie

%% Go through measurements

% Store curves with same conditions
V_i_prev = 0; % to keep track of measurements with same condition
V_1_i_prev = 0; % only used for legend entry for sliding rate
curves_store = []; % temporarily store friction curves over loop
count_store = 0;

for meas_i = 1:length(range_meas) % go through selected measurements

    meas = range_meas(meas_i);

    %% load data

    % find file and get data
    cd ../FT_C-PEEK_15kPa_385degC
    file_find = dir(['*',num2str(serie),'-',num2str(meas),'.txt']);
    file_name = file_find.name; 
    data = readmatrix(file_name);
    cd ../Scripts
    
    %% interpret data
    
    t = data(:,1) - data(1,1); % time [s]
    d = data(:,2) - data(1,2); % displacement [mm]
    f_pull = data(:,3); % pull force [N]
    
    %% compute shear stress

    A = 100*50; % contact area, mm^2      
    tau = f_pull/(2*A)*1e3; % shear stress; 2 slip interfaces, from MPa to kPa

    %% determine start and end points end mean sliding rate

    % velocity from file name
    name_u = find(file_find.name=='u');
    name_u_end = name_u + 1;
    while file_find.name(name_u_end) ~= '_'
        name_u_end = name_u_end + 1;
    end
    vel = str2double(file_find.name(name_u+1:name_u_end-1)); % velocity from file name

    % determine window for smoothening based on velocity from file name
    if vel <= 1
        window = 525;
    elseif vel <= 3
        window = 500;
    elseif vel <= 5
        window = 200;
    elseif vel <= 11
        window = 50;
    elseif vel <= 15
        window = 45;
    elseif vel <= 30
        window = 40;
    elseif vel <= 50
        window = 25;
    else
        window = 15;
    end

    % smoothening data
    d_filt = smoothdata(d,'gaussian',window); % filter data met moving median filter

    v_vec = [0]; % velocity vector
    v_vec_1 = [0]; % velocity vector 
    a_vec = [0]; % acceleration vector
    for i = 2:length(t)-1
        dt_L = t(i) - t(i-1);
        dt_R = t(i+1) - t(i);
        v = (d_filt(i+1)-d_filt(i-1))/(dt_R+dt_L);
        a = (d_filt(i+1)-2*d_filt(i)+d_filt(i-1))/(0.5*(dt_R^2+dt_L^2)); % actually: error due to f' times difference in time step
        v_vec(i) = v;
        a_vec(i) = a;

        % first order derivative
        v_vec_1(i) = (d_filt(i+1)-d_filt(i))/(t(i+1)-t(i));
    end
    v_vec(length(t)) = v;
    v_vec_1(length(t)) = v_vec_1(length(t)-1);
    v_vec_grad = gradient(d_filt(:)) ./ gradient(t(:));
    a_vec(length(t)) = a;

    % smooth data at very low V
    if vel == 1
        a_vec = smoothdata(a_vec,'gaussian',50);
    elseif vel >= 2 && vel < 5
        a_vec = smoothdata(a_vec,'gaussian',10);
    elseif vel == 5
        a_vec = smoothdata(a_vec,'gaussian',5);
    end

    % find global start and end points
    i_glob = 1; % global end range of start point
    while d(i_glob) < 1 % deformation below 1 mm
        i_glob = i_glob + 1;
    end
    
    i_glob_9 = i_glob; % global end range of start point
    while d(i_glob_9) < 9 % deformation around 9 mm
        i_glob_9 = i_glob_9 + 1;
    end
    
    i_glob_start = i_glob; % find around i_glob also global start range for start point
    while t(i_glob_start) > t(i_glob)-(2/vel*60) && i_glob_start > 2 % move 2 mm in time earlier based on velocity in experiment
        i_glob_start = i_glob_start - 1;
    end
            
    i_glob_end = i_glob;
    while t(i_glob_end) < t(i_glob) + d(end)/vel*60 && i_glob_end < length(t)-1 % find index for time after full displacement
        i_glob_end = i_glob_end + 1;
    end
    idx_start = find(a_vec(i_glob_start:i_glob)==max(a_vec(i_glob_start:i_glob)))+i_glob_start-1;
    idx_end = find(a_vec(i_glob_9:i_glob_end)==min(a_vec(i_glob_9:i_glob_end)))+i_glob_9-1;

    %% modulus fit
    % used to correct data
        
    [t,d] = mod_fit_v1(idx_start,idx_end,t,d,tau,mod_coeff,20);

    %% isolate friction curve and smooth

    d_curve = d(idx_start:idx_end);
    tau_curve = tau(idx_start:idx_end);
    
    % small manual adjustments
    if serie == 112 && ismember(meas,[5 9])
        d_curve = d_curve + 0.04;
    elseif serie == 113
        if meas == 1
            d_curve = d_curve + 0.08;
        elseif meas == 2
            d_curve = d_curve + 0.06;
        end
    end
    
    d_curve = smoothdata(d_curve,'gaussian',5); % filter data with moving median filter
    tau_curve = smoothdata(tau_curve,'gaussian',5); % filter data with moving median filter

    %figure(1), hold on
    %plot(d_curve,tau_curve)

    %% determine relaxation curve with residual or yield stress

    % start and end indices for relaxation
    idx_f_rel_start = idx_end - 100;
    idx_f_rel_end = idx_end + 100;
    
    % second derivative shear stress w.r.t. time
    dtau_dt_sec_vec = []; 
    for i_rel = idx_f_rel_start:idx_f_rel_end
        dt = mean([t(i_rel+1)-t(i_rel),t(i_rel)-t(i_rel-1)]);
        dtau_dt_sec = (tau(i_rel+1)-2*tau(i_rel)+tau(i_rel-1))/(dt^2);
        dtau_dt_sec_vec = [dtau_dt_sec_vec, dtau_dt_sec];
    end
    
    idx_rel_start = find(dtau_dt_sec_vec==min(dtau_dt_sec_vec))+idx_f_rel_start-2; % minus 1: to obtain correct pos, extra minus 1: to start before actual sharp decrease
    
    idx_rel_end = idx_rel_start;
    while t(idx_rel_end) < t(idx_rel_start)+rel_time   % relaxation time
        idx_rel_end = idx_rel_end + 1;
    end
    tau_rel = tau(idx_rel_start:idx_rel_end);  % vector with selected range, shear stress
    t_rel = t(idx_rel_start:idx_rel_end)-t(idx_rel_start);  % relaxation time

    %% Compute average curves
    
    V_i = vel; % velocity

    if V_i == 1
        idx_range = 1:100:length(d_curve); 
    elseif V_i == 5
        idx_range = 1:10:length(d_curve);
    else
        idx_range = 1:1:length(d_curve);
    end
        
    if V_i == V_i_prev && meas_i ~= length(range_meas) || isempty(curves_store) % gather friction curves with same sliding rate
        
        curves_store{count_store+1} = [d_curve(idx_range) tau_curve(idx_range)];        
        count_store = count_store + 1;
        V_i_prev = V_i;
                
    else   % compute avg of stored curves and plot, change V_mean_prev and store current curves   
        
        % color codes for plotting
        if V_i_prev == 125
            color_code = [0.0 0.45 0.74];
        elseif V_i_prev == 25
            color_code = [0.49 0.18 0.56];
        elseif V_i_prev == 5
            color_code = [0.47 0.67 0.19];
        elseif V_i_prev == 1
            color_code = [0.93 0.69 0.13];    
        else
            color_code = [0 0 0];
        end
        
        if V_i == V_i_prev && meas_i == length(range_meas)
            curves_store{count_store+1} = [d_curve(idx_range) tau_curve(idx_range)];            
        end
        
        % compute avg for considered velocity
                               
        % find index for longest dataset in stored curves
        j_start = 1;
        j_end = length(curves_store);
        j_max = 1;
        length_max = 1;
        for j = 1:j_end
            if length(curves_store{j}) > length_max
                j_max = j;
                length_max = length(curves_store{j});
            end
        end

        % find start and end indices of basis curve (longest)
        d_base = curves_store{j_max}(:,1);
        k_start = min(find(d_base>0));
        d_end = 10;
        for j = 1:j_end % find shortest distance of curves
            if max(curves_store{j}(:,1)) < d_end
                d_end = max(curves_store{j}(:,1));
            end
        end
        k_end = min(find(d_base>=d_end))-1;
        
        % determine mean curves
        d_mean = zeros(1,k_end-k_start+1);
        tau_mean = zeros(1,k_end-k_start+1);
        std_mean = zeros(1,k_end-k_start+1);
        k = k_start;
        while k <= k_end
            d_k = d_base(k);
            d_mean(k-k_start+1) = d_k;

            % find closest tau values; interpolation
            tau_k = zeros(1,j_end);
            for j = 1:j_end
                if j == j_max
                    tau_k(j) = curves_store{j}(k,2);
                else
                    idx_left = max(find(curves_store{j}(:,1)<d_k));
                    idx_right = min(find(curves_store{j}(:,1)>d_k));
                    if ~isempty(idx_right) && ~isempty(idx_left)

                        % interpolation to find tau_k
                        tau_L = curves_store{j}(idx_left,2); 
                        tau_R = curves_store{j}(idx_right,2);
                        d_L = curves_store{j}(idx_left,1);
                        d_R = curves_store{j}(idx_right,1);
                        tau_k(j-j_start+1) = tau_L + (tau_R-tau_L)*((d_k-d_L)/(d_R-d_L));
                    elseif isempty(idx_left) % account for start of curve
                        tau_k(j-j_start+1) = curves_store{j}(idx_right,2);
                    elseif isempty(idx_right) % account for end of curve
                        tau_k(j-j_start+1) = curves_store{j}(idx_left,2);
                    end
                end
            end

            tau_mean(k-k_start+1) = mean(tau_k);
            std_mean(k-k_start+1) = std(tau_k);
            k = k + 1;
        end
        
        % plot average and plot error band (standard deviation)
        figure(1), hold on
        plt = fill([d_mean, fliplr(d_mean)],[tau_mean-std_mean, fliplr(tau_mean+std_mean)],[0.5 0.5 0.5],'LineStyle','none','facecolor',color_code);
        set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );
        set(plt, 'facealpha', 0.3)
        plot(d_mean,tau_mean,'-k','linewidth',2.0,'color',color_code,'displayname',[num2str(V_i_prev), ' mm/min']);     

        % emtpy storage and store current curves       
        curves_store = [];
        count_store = 1;
        V_i_prev = V_i;
        curves_store{count_store} = [d_curve(idx_range) tau_curve(idx_range)];
    end

    
    %% plot data

    % color codes for plotting
    if V_i  == 125
        color_code = [0.0 0.45 0.74];
        idx_range_rel = 1:1:length(t_rel);
    elseif V_i == 25
        color_code = [0.49 0.18 0.56];
        idx_range_rel = 1:1:length(t_rel);
    elseif V_i_prev == 5
        color_code = [0.47 0.67 0.19];
        idx_range_rel = 1:10:length(t_rel);
    elseif V_i == 1
        color_code = [0.93 0.69 0.13];
        idx_range_rel = 1:10:length(t_rel);
    end
    
    % relaxation
    figure(2), hold on
    plot(t_rel(idx_range_rel),tau_rel(idx_range_rel),'-k','color',color_code)
end

%% Outside the loop

figure(1)
text(1.05,92,'$\tau_{\rm{p}}$','interpreter','latex','fontsize',8)
text(8.7,45,'$\tau_{\infty}$','interpreter','latex','fontsize',8)
leg = legend('box','off','fontsize',8,'interpreter','latex','location','NorthEast','NumColumns',1);
leg.ItemTokenSize = [22,18];
xlabel('Displacement, $d$ [mm]','interpreter','latex','fontsize',8)
ylabel('Shear stress, $\tau$ [kPa]','interpreter','latex','fontsize',8)
set(gca,'fontsize',8);
set(gcf,'Renderer','painters')

% figure size 
width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
set(gcf,'units','centimeters','position',[5,5,width,0.75*width])
axis([0 9.5 0 100])
grid on
box on

% save for PDF (as EPS doesn't support transparency)
set(gcf,'PaperUnits','centimeters','PaperSize',[width 0.75*width])
print('-painters','-dpdf','Fig_friction_curves_CPEEK.pdf')

figure(2), hold on
plt = plot([25.5 25.5 29.5 29.5 25.5],[0.75 3.5 3.5 0.75 0.75],'-k','linewidth',1.0);
set( get( get( plt, 'Annotation'), 'LegendInformation' ), 'IconDisplayStyle', 'off' );
text(27,5.5,'$\tau_{\rm{y}}$','interpreter','latex','fontsize',8)
xlabel('Relaxation time, $t$ [s]','interpreter','latex','fontsize',8)
ylabel('Shear stress, $\tau$ [kPa]','interpreter','latex','fontsize',8)
set(gca,'fontsize',8);
set(gcf,'Renderer','painters')

% figure size 
width = 9; % single column: 9 cm, 1.5 column: 14 cm, double column: 19 cm
set(gcf,'units','centimeters','position',[5,5,width,0.75*width])
axis([0 30 0 40])
grid on
box on

% save for PDF (as EPS doesn't support transparency)
set(gcf,'PaperUnits','centimeters','PaperSize',[width 0.75*width])
print('-painters','-dpdf','Fig_relaxation_curves_CPEEK.pdf')